/*
BromScript - On Runtime Scripting Language.
Copyright (C) 2012 - 2014  Alex Brouwer (Bromvlieg: http://brom.4o3.nl/)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../Managers/Executer.h"
#include "../Managers/Converter.h"
#include "../Objects/Variable.h"
#include "../Objects/Function.h"
#include "../Objects/Table.h"
#include "../Objects/UserdataInstance.h"

#include <cstring>

using namespace Scratch;

namespace BromScript {
	bool CheckIfVarTrue(Variable* var){
		switch (var->Type) {
		case VariableType::Null: return false;
		case VariableType::Number: return *(double*)var->Value >= 0; break;
		case VariableType::String: return strlen(*(CString*)var->Value) > 0; break;
		case VariableType::Table: return ((Table*)var->Value)->Count > 0; break;
		case VariableType::Bool: return var->Value != null; break;
		default: return var->Value != null; break;
		}
	}

	Variable* Executer::PostIncrement(ExecuteData* data) {
		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		Variable* obj = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		if (obj->Type != VariableType::Number){
			data->BromScript->Error("Cannot Post-Increment a non-number");
			return data->BromScript->GC.RegisterVariable();
		}

		(*(double*)obj->Value)++;

		return data->BromScript->GC.RegisterVariable(Converter::ToVariable(obj->GetNumber() - 1));
	}

	Variable* Executer::PreIncrement(ExecuteData* data) {
		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		Variable* obj = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		if (obj->Type != VariableType::Number){
			data->BromScript->Error("Cannot Pre-Increment a non-number");
			return data->BromScript->GC.RegisterVariable();
		}

		(*(double*)obj->Value)++;

		return obj;
	}

	Variable* Executer::PostDecrement(ExecuteData* data) {
		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		Variable* obj = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		if (obj->Type != VariableType::Number){
			data->BromScript->Error("Cannot Post-Decrement a non-number");
			return data->BromScript->GC.RegisterVariable();
		}

		(*(double*)obj->Value)--;

		return data->BromScript->GC.RegisterVariable(Converter::ToVariable(obj->GetNumber() + 1));
	}

	Variable* Executer::PreDecrement(ExecuteData* data) {
		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		Variable* obj = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		if (obj->Type != VariableType::Number){
			data->BromScript->Error("Cannot Pre-Decrement a non-number");
			return data->BromScript->GC.RegisterVariable();
		}

		(*(double*)obj->Value)--;

		return obj;
	}

	Variable* Executer::New(ExecuteData* data) {
		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		Variable* classobj = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		if (classobj->Type == VariableType::Null){
			data->BromScript->Error("Cannot create an instance of null");
			return data->BromScript->GC.RegisterVariable();
		}

		if (classobj->Type != VariableType::Class){
			data->BromScript->Error("Attempting to create a non class!");
			return data->BromScript->GC.RegisterVariable();
		}

		int params = data->Reader->ReadInt();
		List<Variable*> vars;
		for (int i = 0; i < params; i++){
			bool oldcall = data->FuncCallback;
			data->FuncCallback = true;
			vars.Add(data->Function->InternalRun(data));
			data->FuncCallback = oldcall;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return data->Returning;
		}

		List<CString> setskeys;
		List<Variable*> setsvalues;
		int sets = data->Reader->ReadInt();
		for (int i = 0; i < sets; i++){
			setskeys.Add(data->Reader->ReadString());

			bool oldcall = data->FuncCallback;
			data->FuncCallback = true;
			setsvalues.Add(data->Function->InternalRun(data));
			data->FuncCallback = oldcall;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return data->Returning;
		}

		ArgumentData* args = new ArgumentData();
		args->BromScript = data->BromScript;
		args->SetVariableData(&vars);

		Function* func = (Function*)classobj->Value;
		if (func->IsCpp){
			Userdata* ud = null;
			for (int i = 0; i < data->BromScript->RegisteredUserdataTypes.Count; i++){
				if (data->BromScript->RegisteredUserdataTypes[i]->Name == func->Name){
					ud = data->BromScript->RegisteredUserdataTypes[i];
					break;
				}
			}

			if (ud->Ctor == null){
				data->BromScript->Error("Cannot create type, no constructor found!");
				return data->BromScript->GC.RegisterVariable();
			}

			UserdataInstance* udi = new UserdataInstance();
			udi->TypeData = ud;
			udi->Ptr = ud->Ctor(data->BromScript, args);
			udi->CallDTor = true;

			Variable* ret = data->BromScript->GC.RegisterVariable();
			ret->Type = (VariableType::Enum)ud->TypeID;
			ret->Value = udi;

			for (int i = 0; i < sets; i++){
				bool found = false;
				for (int i2 = 0; i2 < ud->Members.Count; i2++){
					if (ud->Members[i2]->Name == setskeys[i]){
						UserdataInstance* udi2 = new UserdataInstance();
						udi2->TypeData = ud->Members[i2];
						udi2->Ptr = (char*)udi->Ptr + ud->Members[i2]->Offset;
						udi2->CallDTor = true;

						Variable* tmpvar = new Variable();
						tmpvar->Type = (VariableType::Enum)udi2->TypeData->TypeID;
						tmpvar->Value = udi2;
						Converter::SetMember(data->BromScript, tmpvar, setsvalues[i], setskeys[i]);
						delete tmpvar;

						found = true;
						break;
					}
				}

				if (!found){
					delete args;
					data->BromScript->Error(CString::Format("Cannot find index %s on object type %s", setskeys[i].str_szBuffer, ud->Name.str_szBuffer));
					return ret;
				}
			}

			delete args;
			return ret;

		}else{
			Table* tbl = new Table(data->BromScript);
			Variable* ret = data->BromScript->GC.RegisterVariable(Converter::ToVariable(tbl));

			int paramloops = args->Count < func->Parameters.Count ? args->Count : func->Parameters.Count;

			List<void*> oldvals;
			for (int i = 0; i < paramloops; i++)
				oldvals.Add(vars[i]->Value);

			func->Run(args, ret);

			for (int i = 0; i < func->Locals->Count; i++){
				CString vkey = func->Locals->GetKeyByIndex(i);
				bool found = false;
				for (int i2 = 0; i2 < paramloops; i2++){
					if (func->Parameters[i2] == vkey){
						if (func->Locals->GetByIndex(i)->Value != oldvals[i2])
							tbl->Set(vkey, func->Locals->GetByIndex(i));

						found = true;
						break;
					}
				}

				if (!found)
					tbl->Set(vkey, func->Locals->GetByIndex(i));
			}

			func->Locals->Clear();

			for (int i = 0; i < sets; i++)
				tbl->Set(setskeys[i], setsvalues[i]);

			tbl->Set("__TYPE", Converter::ToVariable(func->Name.Substring(0, -5)));
			tbl->Get("__TYPE")->ReadOnly = true;

			delete args;
			return ret;
		}
	}

	void Executer::CreateClass(ExecuteData* data) {
		bool islocal = data->Reader->ReadBool();
		CString classname = data->Reader->ReadString();
		List<CString>* paras = data->Reader->ReadStrings();

		int codelen = data->Reader->ReadInt();
		byte* code = data->Reader->ReadBytes(codelen);

		Function* func = new Function(data->BromScript, data->Function->StringTable);
		func->Name = classname + "_CTOR";
		func->Filename = data->Function->Filename;
		func->Parent = data->Function;
		func->Code = code;

		Variable* var = data->BromScript->GC.RegisterVariable();
		var->Value = func;
		var->Type = VariableType::Class;

		for (int i = 0; i < paras->Count; i++)
			func->Parameters.Add(CString(paras->Get(i)));

		delete paras;

		data->Function->SetVar(classname, var, !islocal);
	}

	void Executer::StringTable(ExecuteData* data) {
		int num = data->Reader->ReadShort();
		data->Function->StringTable = new CString[num];
		for (int i = 0; i < num; i++){
			int strlen = data->Reader->ReadShort();
			byte* tmpbuff = data->Reader->ReadBytes(strlen);

			char* destbuff = new char[strlen + 1];
			memcpy(destbuff, tmpbuff, strlen);
			destbuff[strlen] = 0;

			data->Function->StringTable[i].str_szBuffer = destbuff;
			delete[] tmpbuff;
		}
	}

	Variable* Executer::CreateAnonFunction(ExecuteData* data) {
		CString filename = data->Reader->ReadString();
		int codelen = data->Reader->ReadInt();

		Function* func = new Function(data->BromScript, data->Function->StringTable);
		func->Name = "AnonFunction";
		func->Filename = filename;
		func->Parent = data->Function;
		func->Code = data->Reader->ReadBytes(codelen);

		Variable* var = data->BromScript->GC.RegisterVariable();
		var->Value = func;
		var->Type = VariableType::Function;

		List<CString>* params = data->Reader->ReadStrings();
		for (int i = 0; i < params->Count; i++)
			func->Parameters.Add(CString(params->Get(i)));

		delete params;
		return var;
	}

	void Executer::CreateFunction(ExecuteData* data) {
		CString funcname = data->Reader->ReadString();
		CString filename = data->Reader->ReadString();
		int codelen = data->Reader->ReadInt();

		Function* func = new Function(data->BromScript, data->Function->StringTable);
		func->Name = funcname;
		func->Filename = filename;
		func->Parent = data->Function;
		func->Code = data->Reader->ReadBytes(codelen);

		Variable* var = data->BromScript->GC.RegisterVariable();
		var->Value = func;
		var->Type = VariableType::Function;

		List<CString>* params = data->Reader->ReadStrings();
		for (int i = 0; i < params->Count; i++)
			func->Parameters.Add(CString(params->Get(i)));

		delete params;

		List<CString>* keys = data->Reader->ReadStrings();

		bool islocal = data->Reader->ReadBool();
		if (keys->Count == 1){
			data->Function->SetVar(funcname, var, !islocal);
		}else{
			Variable* curvar = data->Function->GetVar(funcname);
			Variable* prevvar = null;

			if (curvar->Type == VariableType::Null){
				data->BromScript->Error("Cannot index a null value");
				delete keys;
				return;
			}

			if (curvar->Type != VariableType::Table){
				data->BromScript->Error("Cannot index a non table value");
				delete keys;
				return;
			}

			for (int i = 1; i < keys->Count; i++){
				if (i == keys->Count - 1){
					if (curvar->Type == VariableType::Null){
						data->BromScript->Error("Cannot index a null value");
						delete keys;
						return;
					}

					if (curvar->Type != VariableType::Table && curvar->Type <= VariableType::Userdata){
						data->BromScript->Error("Cannot index a non table value");
						delete keys;
						return;
					}

					CString endkey = keys->Get(keys->Count - 1);
					if (curvar->Type == VariableType::Table){
						Table* tbl = (Table*)curvar->Value;
						tbl->Set(endkey, var);
						delete keys;
						return;
					}else{
						bool found = false;
						UserdataInstance* udi = (UserdataInstance*)prevvar->Value;
						for (int i2 = 0; i2 < udi->TypeData->Members.Count; i2++){
							if (udi->TypeData->Members[i2]->Name == endkey){
								UserdataInstance* udi2 = new UserdataInstance();
								udi2->TypeData = udi->TypeData->Members[i2];
								udi2->Ptr =(byte*)udi->Ptr + udi2->TypeData->Offset;

								curvar->Value = udi2;
								curvar->Type = (VariableType::Enum)udi2->TypeData->TypeID;
								curvar->IsCpp = true;
								found = true;

								if (!Converter::SetMember(data->BromScript, curvar, var, endkey)){
									data->BromScript->Error("Failed to set member");
									delete keys;
									return;
								}

								delete keys;
								return;
							}
						}

						data->BromScript->Error(CString::Format("Unknown userdata index '%s'", endkey.str_szBuffer));
					}

					delete keys;
					return;
				}

				if (curvar->Type == VariableType::Table){
					Table* CurrentTable = (Table*)curvar->Value;

					prevvar = curvar;
					curvar = CurrentTable->Get(keys->Get(i));
				}else if (curvar->Type > VariableType::Userdata){
					UserdataInstance* udi = (UserdataInstance*)curvar->Value;

					prevvar = curvar;
					curvar = data->BromScript->GC.RegisterVariable();

					bool found = false;
					for (int i2 = 0; i2 < udi->TypeData->Members.Count; i2++){
						if (udi->TypeData->Members[i2]->Name == keys->Get(i)){
							UserdataInstance* udi2 = new UserdataInstance();
							udi2->TypeData = udi->TypeData->Members[i2];
							udi2->Ptr = (byte*)udi->Ptr + udi2->TypeData->Offset;

							curvar->Value = udi2;
							curvar->Type = (VariableType::Enum)udi2->TypeData->TypeID;
							curvar->IsCpp = true;
							found = true;
							break;
						}
					}

					if (!found){
						for (int i2 = 0; i2 < udi->TypeData->Functions.Count(); i2++){
							if (udi->TypeData->Functions.GetKeyByIndex(i2) == keys->Get(i)){
								Function* func = new Function(data->BromScript, data->Function->StringTable);
								func->CppFunc = udi->TypeData->Functions.GetValueByIndex(i2);
								func->IsCpp = true;
								func->Name = keys->Get(i);
								func->Filename = "C++";

								curvar->Type = VariableType::Function;
								curvar->Value = func;

								found = true;
								break;
							}
						}

						if (!found){
							data->BromScript->Error(CString::Format("Unknown userdata index '%s'", keys->Get(i).str_szBuffer));
							delete keys;
							return;
						}
					}
				}else{
					data->BromScript->Error("Cannot index a non table value");
					delete keys;
					return;
				}
			}
		}

		delete keys;
	}
	
	void Executer::SetTableIndex(ExecuteData* data){
		CString tblname = data->Reader->ReadString();
		CString fullvarname = tblname;
		int loops = data->Reader->ReadByte();

		Variable* prevvar = null;
		Variable* curvar = data->Function->GetVar(tblname);

		CString key;

		for (int i = 0; i <= loops; i++){
			bool oldcall = data->FuncCallback;
			data->FuncCallback = true;
			Variable* keyvar = data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return;

			if (i == loops){
				if (curvar->Type > VariableType::Userdata){
					if (!Converter::SetMember(data->BromScript, curvar, keyvar, key)){
						data->BromScript->Error("Failed to set member");
						return;
					}
					return;
				}else if (prevvar->Type == VariableType::Table){
					Table* tbl = (Table*)prevvar->Value;
					if (tbl->Get(key)->ReadOnly){
						data->BromScript->Error(CString::Format("Cannot override '%s' it's read only!", fullvarname.str_szBuffer));
						return;
					}

					if (keyvar->Type == VariableType::Null)
						tbl->Remove(key);
					else
						tbl->Set(key, keyvar);

					return;
				}else{
					data->BromScript->Error(CString::Format("Cannot set index of non-indexable value", fullvarname.str_szBuffer));
					return;
				}
			}

			if (keyvar->Type == VariableType::Bool) key = keyvar->Value == null ? "False" : "True";
			else if (keyvar->Type == VariableType::Number) key = CString(*(double*)keyvar->Value);
			else if (keyvar->Type == VariableType::String) key = *(CString*)keyvar->Value;
			else{
				data->BromScript->Error(CString::Format("Cannot index a table value using keytype %s", Converter::TypeToString(data->BromScript, keyvar).str_szBuffer));
				return;
			}

			if (key.Size() > 0){
				if (curvar->Type == VariableType::Table){
					Table* CurrentTable = (Table*)curvar->Value;

					prevvar = curvar;
					curvar = CurrentTable->Get(key);
				}else if (curvar->Type == VariableType::String){
					if (keyvar->Type != VariableType::Number){
						data->BromScript->Error(CString::Format("Cannot index a string value using keytype %s", Converter::TypeToString(data->BromScript, keyvar).str_szBuffer));
						return;
					}
					
					CString tmpstr = ((CString*)curvar->Value)->str_szBuffer[(int)keyvar->GetNumber()];

					prevvar = curvar;
					curvar = Converter::ToVariable(tmpstr);
				}else if (curvar->Type > VariableType::Userdata){
					UserdataInstance* udi = (UserdataInstance*)curvar->Value;

					prevvar = curvar;
					curvar = data->BromScript->GC.RegisterVariable();

					bool found = false;
					for (int i = 0; i < udi->TypeData->Members.Count; i++){
						if (udi->TypeData->Members[i]->Name == key){
							UserdataInstance* udi2 = new UserdataInstance();
							udi2->TypeData = udi->TypeData->Members[i];
							udi2->Ptr = (byte*)udi->Ptr + udi2->TypeData->Offset;

							curvar->Value = udi2;
							curvar->Type = (VariableType::Enum)udi2->TypeData->TypeID;
							curvar->IsCpp = true;
							found = true;
							break;
						}
					}

					if (!found){
						for (int i = 0; i < udi->TypeData->Functions.Count(); i++){
							if (udi->TypeData->Functions.GetKeyByIndex(i) == key){
								Function* func = new Function(data->BromScript, data->Function->StringTable);
								func->CppFunc = udi->TypeData->Functions.GetValueByIndex(i);
								func->IsCpp = true;
								func->Name = key;
								func->Filename = "C++";

								curvar->Type = VariableType::Function;
								curvar->Value = func;

								found = true;
								break;
							}
						}

						if (!found){
							data->BromScript->Error(CString::Format("Unknown userdata index '%s'", fullvarname.str_szBuffer));
							return;
						}
					}
				}else{
					data->BromScript->Error("Cannot index a non table value");
					return;
				}
			}

			bool exec = data->Reader->ReadBool();
			if (exec){
				if (curvar->Type != VariableType::Function && (prevvar->Type != VariableType::Userdata || ((UserdataInstance*)prevvar->Value)->TypeData->Operators[Misc::ArithmaticFuncs::Call - 100])){
					data->BromScript->Error(CString::Format("Cannot execute a non function value, varname: %s", fullvarname.str_szBuffer));
					return;
				}

				int params = data->Reader->ReadInt();
				List<Variable*> vars;

				for (int i = 0; i < params; i++){
					bool oldcall = data->FuncCallback;
					data->FuncCallback = true;
					vars.Add(data->Function->InternalRun(data));
					data->FuncCallback = oldcall;

					if (data->HasReturnValue || data->Function->ForceReturn)
						return;
				}

				if (curvar->Type > VariableType::Userdata){
					UserdataInstance* udi = (UserdataInstance*)curvar->Value;
					ArgumentData* args = new ArgumentData();
					args->BromScript = data->BromScript;
					args->Caller = data->Function;
					args->SetVariableData(&vars);
					args->ThisObject = prevvar;

					Variable* ret = udi->TypeData->Operators[Misc::ArithmaticFuncs::Call - 100](data->BromScript, args);
					delete args;

					if (ret == null) ret = data->BromScript->GC.RegisterVariable();
					else data->BromScript->GC.RegisterVariable(ret);
						
					Variable* tmp = prevvar;
					prevvar = curvar;
					curvar = ret;
				}else{
					Function* func = (Function*)curvar->Value;
					Variable* tmp = prevvar;
					prevvar = curvar;
					curvar = func->Run(&vars, tmp);

					if (data->HasReturnValue || data->Function->ForceReturn)
						return;
				}
			}

			fullvarname += CString::Format("[%s]", key.str_szBuffer);
		}
	}

	Variable* Executer::GetTableIndex(ExecuteData* data){
		CString tblname = data->Reader->ReadString();
		CString fullvarname = tblname;
		int loops = data->Reader->ReadByte();

		Variable* keyvar = null;
		Variable* prevvar = null;
		Variable* curvar = data->Function->GetVar(tblname);

		for (int i = 0; i < loops; i++){
			bool oldcall = data->FuncCallback;
			data->FuncCallback = true;
			keyvar = data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return data->BromScript->GC.RegisterVariable();

			CString key;
			if (keyvar->Type == VariableType::Bool) key = keyvar->Value == null ? "False" : "True";
			else if (keyvar->Type == VariableType::Number) key = CString(*(double*)keyvar->Value);
			else if (keyvar->Type == VariableType::String) key = *(CString*)keyvar->Value;
			else{
				data->BromScript->Error(CString::Format("Cannot index a table value using keytype %s", Converter::TypeToString(data->BromScript, keyvar).str_szBuffer));
				return data->BromScript->GC.RegisterVariable();
			}

			if (key.Size() > 0){
				if (curvar->Type == VariableType::Table){
					Table* CurrentTable = (Table*)curvar->Value;

					prevvar = curvar;
					curvar = CurrentTable->Get(key);
				}else if (curvar->Type == VariableType::String){
					if (keyvar->Type != VariableType::Number){
						data->BromScript->Error(CString::Format("Cannot index a string value using keytype %s", Converter::TypeToString(data->BromScript, keyvar).str_szBuffer));
						return data->BromScript->GC.RegisterVariable();
					}

					CString tmpstr = ((CString*)curvar->Value)->str_szBuffer[(int)keyvar->GetNumber()];

					prevvar = curvar;
					curvar = Converter::ToVariable(tmpstr);
				}else if (curvar->Type > VariableType::Userdata){
					UserdataInstance* udi = (UserdataInstance*)curvar->Value;

					prevvar = curvar;
					curvar = data->BromScript->GC.RegisterVariable();

					bool found = false;
					for (int i = 0; i < udi->TypeData->Members.Count; i++){
						if (udi->TypeData->Members[i]->Name == key){
							if (udi->TypeData->Members[i]->Getter != null){
								curvar = udi->TypeData->Members[i]->Getter(data->BromScript, (byte*)udi->Ptr + udi->TypeData->Members[i]->Offset);

								if (curvar == null) curvar = data->BromScript->GC.RegisterVariable();
								else data->BromScript->GC.RegisterVariable(curvar);
							}else{
								UserdataInstance* udi2 = new UserdataInstance();
								udi2->TypeData = udi->TypeData->Members[i];
								udi2->Ptr = (byte*)udi->Ptr + udi2->TypeData->Offset;

								curvar->Value = udi2;
								curvar->Type = (VariableType::Enum)udi2->TypeData->TypeID;
								curvar->IsCpp = true;
							}
							found = true;
							break;
						}
					}

					if (!found){
						for (int i = 0; i < udi->TypeData->Functions.Count(); i++){
							if (udi->TypeData->Functions.GetKeyByIndex(i) == key){
								Function* func = new Function(data->BromScript, data->Function->StringTable);
								func->CppFunc = udi->TypeData->Functions.GetValueByIndex(i);
								func->IsCpp = true;
								func->Name = key;
								func->Filename = "C++";

								curvar->Type = VariableType::Function;
								curvar->Value = func;

								found = true;
								break;
							}
						}

						if (!found){
							data->BromScript->Error(CString::Format("Unknown userdata index '%s[\"%s\"]'", fullvarname.str_szBuffer, key.str_szBuffer));
							return data->BromScript->GC.RegisterVariable();
						}
					}
				}else{
					data->BromScript->Error(CString::Format("Cannot index a non table value, %s", fullvarname.str_szBuffer));
					return data->BromScript->GC.RegisterVariable();
				}

				fullvarname += CString::Format("[%s]", key.str_szBuffer);
			}else
				fullvarname += "[]";

			
			bool exec = data->Reader->ReadBool();
			if (exec){
				if (curvar->Type != VariableType::Function && (prevvar->Type != VariableType::Userdata || ((UserdataInstance*)prevvar->Value)->TypeData->Operators[Misc::ArithmaticFuncs::Call - 100])){
					data->BromScript->Error(CString::Format("Cannot execute a non function value, varname: %s", fullvarname.str_szBuffer));
					return data->BromScript->GC.RegisterVariable();
				}

				int params = data->Reader->ReadInt();
				List<Variable*> vars;

				for (int i = 0; i < params; i++){
					bool oldcall = data->FuncCallback;
					data->FuncCallback = true;
					vars.Add(data->Function->InternalRun(data));
					data->FuncCallback = oldcall;

					if (data->HasReturnValue || data->Function->ForceReturn)
						return data->Returning;
				}

				if (keyvar->Type > VariableType::Userdata){
					UserdataInstance* udi = (UserdataInstance*)prevvar->Value;
					ArgumentData* args = new ArgumentData();
					args->BromScript = data->BromScript;
					args->Caller = data->Function;
					args->SetVariableData(&vars);
					args->ThisObject = prevvar;

					Variable* ret = udi->TypeData->Operators[Misc::ArithmaticFuncs::Call - 100](data->BromScript, args);
					delete args;

					if (ret == null) ret = data->BromScript->GC.RegisterVariable();
					else data->BromScript->GC.RegisterVariable(ret);
						
					Variable* tmp = prevvar;
					prevvar = curvar;
					curvar = ret;
				}else{
					Function* func = (Function*)curvar->Value;
					Variable* tmp = prevvar;
					prevvar = curvar;
					curvar = func->Run(&vars, tmp);

					if (data->HasReturnValue || data->Function->ForceReturn)
						return data->BromScript->GC.RegisterVariable();
				}
			}
		}

		return curvar;
	}

	void Executer::SetVar(ExecuteData* data){
		CString key = data->Reader->ReadString();

		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		Variable* var = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		if (data->HasReturnValue || data->Function->ForceReturn)
			return;

		data->Function->SetVar(key, var, true);
	}

	void Executer::LSetVar(ExecuteData* data){
		CString key = data->Reader->ReadString();

		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		Variable* var = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		if (data->HasReturnValue || data->Function->ForceReturn)
			return;

		data->Function->SetVar(key, var, false);
	}

	Variable* Executer::GetVar(ExecuteData* data){
		CString key = data->Reader->ReadString();
		Variable* var = data->Function->GetVar(key);

		return var;
	}

	void Executer::If(ExecuteData* data){
		byte skipnextifs = data->Reader->ReadByte();
		int ifstatmentsize = data->Reader->ReadInt(); // unneeded, remove?

		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		Variable* var = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		if (data->HasReturnValue || data->Breaking || data->Continueing || data->Function->ForceReturn)
			return;

		int codelen = data->Reader->ReadInt();

		if (!CheckIfVarTrue(var)){
			data->Reader->Pos += codelen;
			return;
		}

		int oldpos = data->Reader->Pos;

		oldcall = data->FuncCallback;
		data->FuncCallback = false;
		data->Function->InternalRun(data);
		data->FuncCallback = oldcall;
		if (data->HasReturnValue || data->Breaking || data->Continueing || data->Function->ForceReturn)
			return;

		// incase it returns in the middle, would be bad if we enter at a random point in the code :V
		data->Reader->Pos = oldpos + codelen;
		while(skipnextifs == 1){
			data->Reader->Pos++;
			skipnextifs = data->Reader->ReadByte();

			// skip code data for elseifs
			data->Reader->Pos += data->Reader->ReadInt();
			data->Reader->Pos += data->Reader->ReadInt();
		}

		// only one else..... well lets hope, cus it's only possibru once!
		if (skipnextifs == 2){
			// skip code
			data->Reader->ReadByte();
			data->Reader->Pos += data->Reader->ReadInt();
		}
	}

	void Executer::While(ExecuteData* data){
		int checklen = data->Reader->ReadInt();
		int checkpos = data->Reader->Pos;

		while(true){
			bool oldcall = data->FuncCallback;
			data->FuncCallback = true;
			Variable* var = data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			if (data->HasReturnValue || data->Breaking || data->Continueing || data->Function->ForceReturn)
				return;

			data->Reader->Pos = checkpos + checklen;

			int codelen = data->Reader->ReadInt();
			if (!CheckIfVarTrue(var)){
				data->Reader->Pos = checkpos + checklen + codelen + 5; // +5, End byte and the int len for the bodycode, we should move this up.
				return;
			}

			oldcall = data->FuncCallback;
			data->FuncCallback = false;
			data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return;

			if (data->Breaking){
				data->Breaking = false;
				data->Reader->Pos = checkpos + checklen + codelen + 5; // +5, End byte and the int len for the bodycode, we should move this up.
				return;
			}

			data->Continueing = false;
			data->Reader->Pos = checkpos;// goto if statment
		}
	}

	void Executer::For(ExecuteData* data){
		int initlen = data->Reader->ReadInt();
		int iflen = data->Reader->ReadInt();
		int blocksize = data->Reader->ReadInt();
		int afterlen = data->Reader->ReadInt();

		bool oldcall = data->FuncCallback;
		data->FuncCallback = false;
		data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		int startpos = data->Reader->Pos;
		while(true){
			bool oldcall = data->FuncCallback;
			data->FuncCallback = true;
			Variable* var = data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			data->Reader->Pos = startpos + iflen;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return;

			if (!CheckIfVarTrue(var)){
				data->Reader->Pos = startpos + iflen + blocksize + afterlen;
				return;
			}

			oldcall = data->FuncCallback;
			data->FuncCallback = false;
			data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			data->Reader->Pos = startpos + iflen + blocksize;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return;

			if (data->Breaking){
				data->Breaking = false;
				data->Reader->Pos = startpos + iflen + blocksize + afterlen;
				return;
			}

			oldcall = data->FuncCallback;
			data->FuncCallback = false;
			data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			data->Reader->Pos = startpos + iflen + blocksize + afterlen;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return;

			data->Continueing = false;
			data->Reader->Pos = startpos; // back to the depths we go!
		}
	}

	void Executer::ForEach(ExecuteData* data){
		CString key = data->Reader->ReadString();
		CString value = data->Reader->ReadString();

		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		Variable* var = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		if (data->HasReturnValue || data->Function->ForceReturn)
			return;

		if (var->Type == VariableType::Null){
			data->BromScript->Error(CString::Format("Cannot foreach a null value! foreach (%s, %s, NULL)", key.str_szBuffer, value.str_szBuffer));
			return;
		}

		if (var->Type != VariableType::Table && var->Type < VariableType::Userdata){
			data->BromScript->Error(CString::Format("Cannot foreach a non Table value! foreach (%s, %s, NULL)", key.str_szBuffer, value.str_szBuffer));
			return;
		}

		int codelen = data->Reader->ReadInt();
		int codepos = data->Reader->Pos;

		Variable* oldkey = data->Function->GetVar(key);
		Variable* oldvalue = data->Function->GetVar(value);

		if (var->Type == VariableType::Table){
			Table* tbl = (Table*)var->Value;
			int curpos = tbl->GetNextIndex(0);
			while(curpos != -1){
				Variable* valvar = tbl->GetByIndex(curpos);

				Variable* keyvar = data->BromScript->GC.RegisterVariable();
				keyvar->Type = VariableType::String;
				keyvar->Value = Converter::StringToPointer(tbl->GetKeyByIndex(curpos));

				data->Function->SetVar(key, keyvar, false);
				data->Function->SetVar(value, valvar, false);

				bool oldcall = data->FuncCallback;
				data->FuncCallback = false;
				data->Function->InternalRun(data);
				data->FuncCallback = oldcall;

				if (data->HasReturnValue || data->Function->ForceReturn)
					return;

				if (data->Breaking){
					data->Breaking = false;
					data->Reader->Pos = codepos + codelen;
					return;
				}

				data->Continueing = false;
				data->Reader->Pos = codepos; // back to the depths we go!

				curpos = tbl->GetNextIndex(curpos + 1);
			}
		}else{
			UserdataInstance* udi = (UserdataInstance*)var->Value;

			for (int i = 0; i < udi->TypeData->Members.Count + udi->TypeData->Functions.Count(); i++){
				if (i < udi->TypeData->Members.Count){
					Variable* valvar = data->BromScript->GC.RegisterVariable();
					Variable* keyvar = Converter::ToVariable(udi->TypeData->Members[i]->Name);

					if (udi->TypeData->Members[i]->Getter != null){
						valvar = udi->TypeData->Members[i]->Getter(data->BromScript, (byte*)udi->Ptr + udi->TypeData->Members[i]->Offset);

						if (valvar == null) valvar = data->BromScript->GC.RegisterVariable();
						else data->BromScript->GC.RegisterVariable(valvar);
					}else{
						UserdataInstance* udi2 = new UserdataInstance();
						udi2->TypeData = udi->TypeData->Members[i];
						udi2->Ptr = (byte*)udi->Ptr + udi2->TypeData->Offset;

						valvar->Value = udi2;
						valvar->Type = (VariableType::Enum)udi2->TypeData->TypeID;
						valvar->IsCpp = true;
					}

					data->Function->SetVar(key, keyvar, false);
					data->Function->SetVar(value, valvar, false);
				}else{
					int gi = i - udi->TypeData->Members.Count;
					Variable* keyvar = Converter::ToVariable(udi->TypeData->Functions.GetKeyByIndex(gi));
					Variable* valvar = Converter::ToVariable(data->BromScript, udi->TypeData->Functions.GetKeyByIndex(gi), udi->TypeData->Functions.GetValueByIndex(gi));

					data->Function->SetVar(key, keyvar, false);
					data->Function->SetVar(value, valvar, false);
				}

				bool oldcall = data->FuncCallback;
				data->FuncCallback = false;
				data->Function->InternalRun(data);
				data->FuncCallback = oldcall;

				if (data->HasReturnValue || data->Function->ForceReturn)
					return;

				if (data->Breaking){
					data->Breaking = false;
					data->Reader->Pos = codepos + codelen;
					return;
				}

				data->Continueing = false;
				data->Reader->Pos = codepos; // back to the depths we go!
			}
		}

		data->Function->SetVar(key, oldkey, false);
		data->Function->SetVar(value, oldvalue, false);

		data->Reader->Pos = codepos + codelen;
	}

	void Executer::Else(ExecuteData* data){
		// not realy needed anymore i guess. could be removed.
		int codelen = data->Reader->ReadInt();
		int codepos = data->Reader->Pos;

		bool oldcall = data->FuncCallback;
		data->FuncCallback = false;
		data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		data->Reader->Pos = codepos + codelen; // back to the depths we go!
	}

	bool ShouldDoArithmatic(int nf){
		return nf >= Misc::ArithmaticFuncs::Add && nf <= Misc::ArithmaticFuncs::Modulo;
	}

	Variable* Executer::Merge(ExecuteData* data){
		int spos = data->Reader->Pos;
		int mergesize = data->Reader->ReadInt();

		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		Variable* var = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		if (data->HasReturnValue || data->Function->ForceReturn)
			return data->Returning;

		if (var->Type >= VariableType::Userdata)
			var = Converter::MemberToVariable(data->BromScript, var);

		Misc::ExecFuncs nf = (Misc::ExecFuncs)data->Function->Code[data->Reader->Pos];

		if (nf == Misc::ExecFuncs::MergeEnd){
			data->Reader->Pos++;
			return var;
		}

		while(ShouldDoArithmatic((int)nf)){
			data->Reader->Pos++;

			// dont resume when left side is false at AND op
			if (nf == Misc::ArithmaticFuncs::And && !CheckIfVarTrue(var)){
				Variable* ret = data->BromScript->GC.RegisterVariable();
				ret->Type = VariableType::Bool;
				ret->Value = null;
				data->Reader->Pos = spos + mergesize - 1;
				return ret;
			}

			// dont resume when left side is true at OR op
			if (nf == Misc::ArithmaticFuncs::Or && CheckIfVarTrue(var)){
				Variable* ret = data->BromScript->GC.RegisterVariable();
				ret->Type = VariableType::Bool;
				ret->Value = (void*)1;
				data->Reader->Pos = spos + mergesize - 1;
				return ret;
			}

			oldcall = data->FuncCallback;
			data->FuncCallback = true;
			Variable* var2 = data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			if (data->HasReturnValue || data->Breaking || data->Continueing || data->Function->ForceReturn)
				return data->Returning;

			if (var2->Type >= VariableType::Userdata)
				var2 = Converter::MemberToVariable(data->BromScript, var2);


			var = Executer::Arithmatic(data, var, var2, (Misc::ArithmaticFuncs)nf);

			nf = (Misc::ExecFuncs)data->Function->Code[data->Reader->Pos];
		}

		if (nf == Misc::ExecFuncs::MergeEnd){
			data->Reader->Pos++;
			return var;
		}

		return var;
	}

	Variable* Executer::ExecuteFunction(ExecuteData* data) {
		CString name = data->Reader->ReadString();
		int params = data->Reader->ReadInt();

		List<Variable*> vars;
		for (int i = 0; i < params; i++){
			bool oldcall = data->FuncCallback;
			data->FuncCallback = true;
			vars.Add(data->Function->InternalRun(data));
			data->FuncCallback = oldcall;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return data->Returning;
		}

		Variable* var = data->Function->GetVar(name);

		if (var == null){
			data->BromScript->Error("Trying to execute a null value!");
			return data->BromScript->GC.RegisterVariable();
		}

		if (var->Type != VariableType::Function){
			if (var->Type > VariableType::Userdata){
				UserdataInstance* udi = (UserdataInstance*)var->Value;
				if (udi->TypeData->Operators[Misc::ArithmaticFuncs::Call - 100] != null){
					ArgumentData* args = new ArgumentData();
					args->BromScript = data->BromScript;
					args->Caller = data->Function;
					args->SetVariableData(&vars);
					args->ThisObject = var;

					Variable* ret = udi->TypeData->Operators[Misc::ArithmaticFuncs::Call - 100](data->BromScript, args);
					delete args;

					if (ret == null) ret = data->BromScript->GC.RegisterVariable();
					else data->BromScript->GC.RegisterVariable(ret);

					return ret;
				}
			}

			data->BromScript->Error(CString::Format("Trying to execute a non function! Varname: %s", name.str_szBuffer));
			return data->BromScript->GC.RegisterVariable();
		}

		Function* func = (Function*)var->Value;
		Variable* ret = func->Run(&vars);
		return ret;
	}

	Variable* Executer::Return(ExecuteData* data) {
		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		data->Returning = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		data->HasReturnValue = true;
		return data->Returning;
	}

	Variable* Executer::GetCount(ExecuteData* data) {
		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		Variable* ret = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		int len = 0;
		if (ret->Type == VariableType::String){
			len = ((CString*)ret->Value)->Size();
		}else if (ret->Type == VariableType::Table){
			len = ret->GetTable()->Count;
		}else{
			data->BromScript->Error(CString::Format("Trying to get length of type '%s'", Converter::TypeToString(data->BromScript, ret)));
			return data->BromScript->GC.RegisterVariable();
		}

		Variable* var = data->BromScript->GC.RegisterVariable();
		var->Value = Converter::NumberToPointer(len);
		var->Type = VariableType::Number;
		return var;
	}

	Variable* Executer::GetBool(ExecuteData* data) {
		byte b = data->Reader->ReadByte();

		if (b == 2)
			return data->BromScript->GC.RegisterVariable();

		Variable* var = data->BromScript->GC.RegisterVariable();
		var->Value = Converter::BoolToPointer(b == 1);
		var->Type = VariableType::Bool;
		return var;
	}

	Variable* Executer::GetString(ExecuteData* data) {
		Variable* var = data->BromScript->GC.RegisterVariable();
		var->Value = new CString(data->Reader->ReadString());
		var->Type = VariableType::String;
		return var;
	}

	Variable* Executer::GetNumber(ExecuteData* data) {
		Variable* var = data->BromScript->GC.RegisterVariable();
		var->Value = Converter::NumberToPointer(data->Reader->ReadDouble());
		var->Type = VariableType::Number;
		return var;
	}

	Variable* Executer::GetTable(ExecuteData* data) {
		int varcount = data->Reader->ReadInt();

		Variable* var = data->BromScript->GC.RegisterVariable();
		Table* tbl = new Table(data->BromScript);
		var->Value = tbl;
		var->Type = VariableType::Table;

		int curindex = 0;
		for (int i = 0; i < varcount; i++){
			CString key;
			if (data->Reader->ReadBool()){
				key = data->Reader->ReadString();
			}else{
				key = CString((double)curindex++);
			}

			bool oldcall = data->FuncCallback;
			data->FuncCallback = true;
			Variable* ret = data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			tbl->Set(key, ret);
		}

		return var;
	}

	Variable* Executer::GetEnum(ExecuteData* data) {
		int varcount = data->Reader->ReadInt();

		Variable* var = data->BromScript->GC.RegisterVariable();
		Table* tbl = new Table(data->BromScript);
		var->Value = tbl;
		var->Type = VariableType::Table;

		int curindex = -1;
		for (int i = 0; i < varcount; i++){
			CString key = data->Reader->ReadString();

			if (data->Reader->ReadBool()){
				bool oldcall = data->FuncCallback;
				data->FuncCallback = true;
				Variable* ret = data->Function->InternalRun(data);
				data->FuncCallback = oldcall;

				if (ret == null || ret->Type != VariableType::Number){
					data->BromScript->Error(CString::Format("An enum may only contain numbers, and %s is not a number! Key: %s", Converter::TypeToString(data->BromScript, ret).str_szBuffer, key.str_szBuffer));
					return null;
				}

				curindex = (int)ret->GetNumber();
			}else{
				curindex++;
			}

			tbl->Set(key, Converter::ToVariable(curindex));
		}

		return var;
	}

	void Arithmatic_DivideNumber(Variable* ret, Variable* a, Variable* b){
		ret->Type = VariableType::Number;
		ret->Value = Converter::NumberToPointer(*(double*)a->Value / *(double*)b->Value);
	}

	void Arithmatic_MultiplyNumber(Variable* ret, Variable* a, Variable* b){
		ret->Type = VariableType::Number;
		ret->Value = Converter::NumberToPointer(*(double*)a->Value * *(double*)b->Value);
	}

	void Arithmatic_MultiplyString(Variable* ret, Variable* num, Variable* string){
		CString strtoadd(*(CString*)string->Value);
		CString* str = new CString();
		int loops = (int)*(double*)num->Value;

		if (loops <= 0){
			ret->Type = VariableType::String;
			ret->Value = Converter::StringToPointer("");
			return;
		}

		int size = strtoadd.Size();
		int totalsize = loops * size + (int)((*(double*)num->Value - loops) * size);
		char* newp = new char[totalsize + 1];

		str->str_szBuffer = newp;

		for (int i = 0; i < loops; i++)
			memcpy(newp + i * size, strtoadd.str_szBuffer, size);

		if (*(double*)num->Value - loops > 0)
			memcpy(newp + loops * size, strtoadd.str_szBuffer, (int)((*(double*)num->Value - loops) * size));

		newp[totalsize] = null;

		ret->Type = VariableType::String;
		ret->Value = str;
	}

	void Arithmatic_SubstractNumber(Variable* ret, Variable* a, Variable* b){
		ret->Type = VariableType::Number;
		ret->Value = Converter::NumberToPointer(*(double*)a->Value - *(double*)b->Value);
	}

	void Arithmatic_DivideString(Instance* bs, Variable* ret, Variable* a, Variable* b){
		CString str = *(CString*)a->Value;
		Table* tbl = new Table(bs);

		int strlen = str.Size();
		int partlen = (int)(strlen / *(double*)b->Value);

		ret->Type = VariableType::Table;
		ret->Value = tbl;

		if (partlen <= 0)
			return;

		for (int i = 0; i < strlen; i += partlen)
			tbl->Set(CString((double)i), Converter::ToVariable(str.Substring(i, strlen - i < partlen ? strlen - i : partlen)));
	}

	void Arithmatic_AddNumber(Variable* ret, Variable* a, Variable* b){
		ret->Type = VariableType::Number;
		ret->Value = Converter::NumberToPointer(*(double*)a->Value + *(double*)b->Value);
	}

	void Arithmatic_AddString(Variable* ret, Variable* a, Variable* b){
		ret->Type = VariableType::String;
		ret->Value = Converter::StringToPointer(*(CString*)a->Value + *(CString*)b->Value);
	}

	void Arithmatic_AddNumberToString(Variable* ret, Variable* a, Variable* b){
		ret->Type = VariableType::String;
		ret->Value = Converter::StringToPointer(*(CString*)b->Value + CString(*(double*)a->Value));
	}

	void Arithmatic_AddNumberToString2(Variable* ret, Variable* a, Variable* b){
		ret->Type = VariableType::String;
		ret->Value = Converter::StringToPointer(CString::Format("%f%s", *(double*)a->Value, ((CString*)b->Value)->str_szBuffer));
	}

	Variable* Executer::Arithmatic(ExecuteData* data, Variable* a, Variable* b, Misc::ArithmaticFuncs af){
		Variable* ret;

		if (a->Type > VariableType::Userdata || b->Type > VariableType::Userdata){
			UserdataInstance* udi;

			if (a->Type > VariableType::Userdata) udi = (UserdataInstance*)a->Value;
			else udi = (UserdataInstance*)b->Value;

			if (udi->TypeData->Operators[af - 100] != null){
				ArgumentData args;
				args.AddVariable(a);
				args.AddVariable(b);

				ret = udi->TypeData->Operators[af - 100](data->BromScript, &args);
				if (ret == null) ret = data->BromScript->GC.RegisterVariable();
				else data->BromScript->GC.RegisterVariable(ret);

				return ret;
			}else{
				data->BromScript->Error(CString::Format("Invalid arithmatic function!, trying to do %s %s %s", Converter::TypeToString(data->BromScript, a).str_szBuffer, Converter::ArithmaticToString(af).str_szBuffer, Converter::TypeToString(data->BromScript, b).str_szBuffer));
				return data->BromScript->GC.RegisterVariable();
			}
		}

		Variable* bsobj = null;
		if (a->Type == VariableType::Table && a->GetTable()->Get("__TYPE")->Type != VariableType::Null) bsobj = a;
		else if (b->Type == VariableType::Table && b->GetTable()->Get("__TYPE")->Type != VariableType::Null) bsobj = b;

		if (bsobj != null){
			Variable* afunc = bsobj->GetTable()->Get(CString::Format("__%s", Converter::ArithmaticToString(af).str_szBuffer));

			if (afunc->Type == VariableType::Null){
				data->BromScript->Error(CString::Format("Invalid arithmatic function!, trying to do %s %s %s", Converter::TypeToString(data->BromScript, a).str_szBuffer, Converter::ArithmaticToString(af).str_szBuffer, Converter::TypeToString(data->BromScript, b).str_szBuffer));
				return data->BromScript->GC.RegisterVariable();
			}

			if (afunc->Type != VariableType::Function){
				data->BromScript->Error(CString::Format("Invalid arithmatic function!, trying to call a non-function while trying to do %s %s %s", Converter::TypeToString(data->BromScript, a).str_szBuffer, Converter::ArithmaticToString(af).str_szBuffer, Converter::TypeToString(data->BromScript, b).str_szBuffer));
				return data->BromScript->GC.RegisterVariable();
			}

			ArgumentData args;
			args.AddVariable(a);
			args.AddVariable(b);

			Variable* ret = afunc->GetFunction()->Run(&args, bsobj);

			return ret;
		}

		ret = data->BromScript->GC.RegisterVariable();

		switch(af){
		case Misc::ArithmaticFuncs::Add:
			if (a->Type == VariableType::Number){
				if (b->Type == VariableType::Number) Arithmatic_AddNumber(ret, a, b);
				if (b->Type == VariableType::String) Arithmatic_AddNumberToString2(ret, a, b);
			}else if (a->Type == VariableType::String){
				if (b->Type == VariableType::String) Arithmatic_AddString(ret, a, b);
				if (b->Type == VariableType::Number) Arithmatic_AddNumberToString(ret, b, a);
			}
			break;

		case Misc::ArithmaticFuncs::Substract:
			if (a->Type == VariableType::Number){
				if (b->Type == VariableType::Number) Arithmatic_SubstractNumber(ret, a, b);
			}
			break;

		case Misc::ArithmaticFuncs::Divide:
			if (a->Type == VariableType::Number){
				if (b->Type == VariableType::Number) Arithmatic_DivideNumber(ret, a, b);
			}else if (a->Type == VariableType::String){
				if (b->Type == VariableType::Number) Arithmatic_DivideString(data->BromScript, ret, a, b);
			}
			break;

		case Misc::ArithmaticFuncs::Multiply:
			if (a->Type == VariableType::Number){
				if (b->Type == VariableType::Number) Arithmatic_MultiplyNumber(ret, a, b);
				else if (b->Type == VariableType::String) Arithmatic_MultiplyString(ret, a, b);
			}else if (a->Type == VariableType::String){
				if (b->Type == VariableType::Number) Arithmatic_MultiplyString(ret, b, a); // rotated a and b, usage num and string
			}
			break;

		case Misc::ArithmaticFuncs::NotEqual: // we reverse this later on, saves code
		case Misc::ArithmaticFuncs::Equal:
			ret->Type = VariableType::Bool;
			if (a->Type == VariableType::Bool && b->Type == VariableType::Null && a->Value == null) ret->Value = Converter::BoolToPointer(true);
			else if (b->Type == VariableType::Bool && a->Type == VariableType::Null && b->Value != null) ret->Value = Converter::BoolToPointer(true);
			else if (a->Type != b->Type) ret->Value = Converter::BoolToPointer(false);
			else if (a->Type == VariableType::Bool) ret->Value = Converter::BoolToPointer(a->Value == b->Value);
			else if (a->Type == VariableType::Number) ret->Value = Converter::BoolToPointer(*(double*)a->Value == *(double*)b->Value);
			else if (a->Type == VariableType::String) ret->Value = Converter::BoolToPointer(*(CString*)a->Value == *(CString*)b->Value);
			else ret->Value = Converter::BoolToPointer(a->Value == b->Value);
			break;

		case Misc::ArithmaticFuncs::GreaterThan:
			ret->Type = VariableType::Bool;
			if (a->Type != b->Type || a->Type != VariableType::Number){
				ret->Value = Converter::BoolToPointer(false);
			}else{
				ret->Value = Converter::BoolToPointer(*(double*)a->Value > *(double*)b->Value);
			}
			break;

		case Misc::ArithmaticFuncs::GreaterOrEqual:
			ret->Type = VariableType::Bool;
			if (a->Type != b->Type || a->Type != VariableType::Number){
				ret->Value = Converter::BoolToPointer(false);
			}else{
				ret->Value = Converter::BoolToPointer(*(double*)a->Value >= *(double*)b->Value);
			}
			break;

		case Misc::ArithmaticFuncs::LessThan:
			ret->Type = VariableType::Bool;
			if (a->Type != b->Type || a->Type != VariableType::Number){
				ret->Value = Converter::BoolToPointer(false);
			}else{
				ret->Value = Converter::BoolToPointer(*(double*)a->Value < *(double*)b->Value);
			}
			break;

		case Misc::ArithmaticFuncs::LessOrEqual:
			ret->Type = VariableType::Bool;
			if (a->Type != b->Type || a->Type != VariableType::Number){
				ret->Value = Converter::BoolToPointer(false);
			}else{
				ret->Value = Converter::BoolToPointer(*(double*)a->Value <= *(double*)b->Value);
			}
			break;

		case Misc::ArithmaticFuncs::Modulo:
			if (a->Type == VariableType::Number && b->Type == VariableType::Number){
				ret->Type = VariableType::Number;
				ret->Value = Converter::NumberToPointer(fmod((double)*(double*)a->Value, *(double*)b->Value));
			}else if (a->Type == VariableType::String && b->Type == VariableType::Table){
				ret->Type = VariableType::String;

				Table* tbl = b->GetTable();
				CString strRet;

				char* szOffset = ((CString*)a->Value)->str_szBuffer;
				char* szOffsetPrev = szOffset;

				int tblpos = tbl->GetNextIndex(0);
				do {
					szOffset = strstr(szOffset, "%");

					if(szOffset != NULL) {
						if (szOffsetPrev != szOffset && szOffset[-1] == '\\'){ // -1 to go back, this is allowed, however, it feels like YOLO.
							szOffset++;
						}else{
							strRet.AppendToBuffer(szOffsetPrev, szOffset - szOffsetPrev);
							strRet += tbl->GetByIndex(tblpos)->ToString();

							szOffset++;
							szOffsetPrev = szOffset;

							tblpos = tbl->GetNextIndex(++tblpos);
						}
					} else {
						strRet.AppendToBuffer(szOffsetPrev, strlen(szOffsetPrev));
					}
				} while(szOffset != NULL);

				ret->Value = Converter::StringToPointer(strRet.Replace("\\%", "%"));
			}
			break;

		case Misc::ArithmaticFuncs::And:
			ret->Type = VariableType::Bool;
			ret->Value = Converter::BoolToPointer(CheckIfVarTrue(a) && CheckIfVarTrue(b));
			break;

		case Misc::ArithmaticFuncs::Or:
			ret->Type = VariableType::Bool;
			ret->Value = Converter::BoolToPointer(CheckIfVarTrue(a) || CheckIfVarTrue(b));
			break;

		case Misc::ArithmaticFuncs::BitwiseLeft:
			if (a->Type == VariableType::Number && b->Type == VariableType::Number){
				ret->Type = VariableType::Number;
				ret->Value = Converter::NumberToPointer((int)*(double*)a->Value << (int)*(double*)b->Value);
			}
			break;

		case Misc::ArithmaticFuncs::BitwiseRight:
			if (a->Type == VariableType::Number && b->Type == VariableType::Number){
				ret->Type = VariableType::Number;
				ret->Value = Converter::NumberToPointer((int)*(double*)a->Value >> (int)*(double*)b->Value);
			}
			break;

		case Misc::ArithmaticFuncs::BitwiseOr:
			if (a->Type == VariableType::Number && b->Type == VariableType::Number){
				ret->Type = VariableType::Number;
				ret->Value = Converter::NumberToPointer((int)*(double*)a->Value | (int)*(double*)b->Value);
			}
			break;

		case Misc::ArithmaticFuncs::BitwiseAnd:
			if (a->Type == VariableType::Number && b->Type == VariableType::Number){
				ret->Type = VariableType::Number;
				ret->Value = Converter::NumberToPointer((int)*(double*)a->Value & (int)*(double*)b->Value);
			}
			break;
		}

		if (af == Misc::ArithmaticFuncs::NotEqual) // just rotate around equal, less code, same outcome.
			ret->Value = Converter::BoolToPointer(ret->Value == null);

		if (ret->Type == VariableType::Null)
			data->BromScript->Error(CString::Format("Invalid arithmatic function!, trying to do %s %s %s", Converter::TypeToString(data->BromScript, a).str_szBuffer, Converter::ArithmaticToString(af).str_szBuffer, Converter::TypeToString(data->BromScript, b).str_szBuffer));

		return ret;
	}
}